@@ -1,4 +1,5 @@
-## Copyright (c) 2011, Noel O'Boyle
+#-*. coding: utf-8 -*-
+## Copyright (c) 2011, Noel O'Boyle; 2012, Adrià Cereto-Massagué
 ## All rights reserved.
 ##
 ##  This file is part of Cinfony.
@@ -25,22 +26,25 @@ if sys.platform[:3] == "cli":
     import clr
     clr.AddReference('System.Windows.Forms')
     clr.AddReference('System.Drawing')
-    clr.AddReferenceToFileAndPath(_indigonet + "\\indigo-cs.dll")
-    clr.AddReferenceToFileAndPath(_indigonet + "\\indigo-renderer-cs.dll")
+    clr.AddReferenceToFileAndPath(_indigonet + "\\indigo-dotnet.dll")
+    clr.AddReferenceToFileAndPath(_indigonet + "\\indigo-inchi-dotnet.dll")
+    clr.AddReferenceToFileAndPath(_indigonet + "\\indigo-renderer-dotnet.dll")
     from System.Windows.Forms import (
         Application, DockStyle, Form, PictureBox, PictureBoxSizeMode
         )
-    from System.Drawing import Image, Size        
+    from System.Drawing import Image, Size
 elif sys.platform[:4] == "java":
     import java, javax
-    
+
 if sys.platform[:3] == "cli" or sys.platform[:4] == "java":
-    from com.ggasoftware.indigo import Indigo, IndigoException, IndigoRenderer
+    from com.ggasoftware.indigo import Indigo, IndigoException, IndigoRenderer, IndigoInchi
 else:
     from indigo import Indigo, IndigoException
     from indigo_renderer import IndigoRenderer
-    
+    from indigo_inchi import IndigoInchi
+
 indigo = Indigo()
+indigoInchi = IndigoInchi(indigo)
 
 # PIL and Tkinter
 try:
@@ -55,12 +59,13 @@ fps = ["sim", "sub", "sub-res", "sub-tau", "full"]
 
 _formats = {'smi': "SMILES", 'can': "Canonical SMILES", "rdf": "MDL RDF file",
             'mol': "MDL MOL file", 'sdf': "MDL SDF file",
-            'cml': "Chemical Markup Language"}
+            'cml': "Chemical Markup Language",
+            'inchi': "InChI", 'inchikey': "InChIKey"}
 informats = dict([(_x, _formats[_x]) for _x in ['mol', 'sdf', 'rdf', 'smi',
-                                                'cml']])
+                                                'cml', 'inchi']])
 """A dictionary of supported input formats"""
 outformats = dict([(_x, _formats[_x]) for _x in ['mol', 'sdf', 'smi', 'can',
-                                                 'cml']])
+                                                 'cml', 'inchi', 'inchikey']])
 """A dictionary of supported output formats"""
 
 def readfile(format, filename):
@@ -74,10 +79,10 @@ def readfile(format, filename):
     You can access the first molecule in a file using the next() method
     of the iterator:
         mol = readfile("smi", "myfile.smi").next()
-        
+
     You can make a list of the molecules in a file using:
         mols = list(readfile("smi", "myfile.smi"))
-        
+
     You can iterate over the molecules in a file as shown in the
     following code snippet:
     >>> atomtotal = 0
@@ -104,7 +109,7 @@ def readfile(format, filename):
         def rdf_reader():
             for mol in iterator:
                 yield Molecule(mol)
-        return rdf_reader()    
+        return rdf_reader()
     elif format=="mol":
         def mol_reader():
             yield Molecule(indigo.loadMoleculeFromFile(filename))
@@ -120,8 +125,8 @@ def readfile(format, filename):
         def cml_reader():
             for mol in iterator:
                 yield Molecule(mol)
-        return cml_reader()    
-    
+        return cml_reader()
+
     else:
         raise ValueError, "%s is not a recognised Indigo format" % format
 
@@ -139,12 +144,13 @@ def readstring(format, string):
     >>> len(mymol.atoms)
     5
     """
-    format = format.lower()    
+    format = format.lower()
     if format not in informats:
         raise ValueError,"%s is not a recognised Indigo format" % format
 
+    module = indigo if format != "inchi" else indigoInchi
     try:
-        mol = indigo.loadMolecule(string)
+        mol = module.loadMolecule(string)
     except IndigoException:
         raise IOError, "Failed to convert '%s' to format '%s'" % (
             string, format)
@@ -154,7 +160,7 @@ def readstring(format, string):
 
 class Outputfile(object):
     """Represent a file to which *output* is to be sent.
-   
+
     Required parameters:
        format - see the outformats variable for a list of available
                 output formats
@@ -163,7 +169,7 @@ class Outputfile(object):
     Optional parameters:
        overwite -- if the output file already exists, should it
                    be overwritten? (default is False)
-                   
+
     Methods:
        write(molecule)
        close()
@@ -183,10 +189,10 @@ class Outputfile(object):
             self._writer.cmlHeader()
         elif self.format == "rdf":
             self._writer.rdfHeader()
-    
+
     def write(self, molecule):
         """Write a molecule to the output file.
-        
+
         Required parameters:
            molecule
         """
@@ -216,28 +222,28 @@ class Molecule(object):
 
     Required parameter:
        Mol -- an Indigo Mol or any type of cinfony Molecule
-      
+
     Attributes:
        atoms, data, molwt, title
-    
+
     Methods:
        addh(), calcfp(), draw(), localopt(), removeh(),
-       write() 
-      
+       write()
+
     The underlying Indigo Molecule can be accessed using the attribute:
        Mol
     """
     _cinfony = True
-    
+
     def __init__(self, Mol):
         if hasattr(Mol, "_cinfony"):
             a, b = Mol._exchange
             if a == 0:
                 molecule = readstring("smi", b)
             else:
-                molecule = readstring("mol", b)            
+                molecule = readstring("mol", b)
             Mol = molecule.Mol
-            
+
         self.Mol = Mol
 
     @property
@@ -262,14 +268,14 @@ class Molecule(object):
     def addh(self):
         """Add hydrogens."""
         self.Mol.unfoldHydrogens()
-        
+
     def removeh(self):
         """Remove hydrogens."""
         self.Mol.foldHydrogens()
-        
+
     def write(self, format="smi", filename=None, overwrite=False):
         """Write the molecule to a file or return a string.
-        
+
         Optional parameters:
            format -- see the informats variable for a list of available
                      output formats (default is "smi")
@@ -293,6 +299,10 @@ class Molecule(object):
             result = self.Mol.canonicalSmiles()
         elif format=="mol":
             result = self.Mol.molfile()
+        elif format=="inchi":
+            result = indigoInchi.getInchi(self.Mol)
+        elif format=="inchikey":
+            result = indigoInchi.getInchiKey(self.write("inchi"))
         elif format=="cml":
             result = self.Mol.cml()
         elif format=="sdf":
@@ -311,7 +321,7 @@ class Molecule(object):
 
     def __iter__(self):
         """Iterate over the Atoms of the Molecule.
-        
+
         This allows constructions such as the following:
            for atom in mymol:
                print atom
@@ -344,7 +354,7 @@ class Molecule(object):
 
     def calcfp(self, fptype="sim"):
         """Calculate a molecular fingerprint.
-        
+
         Optional parameters:
            fptype -- the fingerprint type (default is "sim"). See the
                      fps variable for a list of of available fingerprint
@@ -390,9 +400,9 @@ class Molecule(object):
                 filedes = None
             else:
                 filedes, filename = tempfile.mkstemp()
-                
+
             renderer.renderToFile(mol, filename)
-                
+
             if show:
                 if sys.platform[:4] == "java":
                     image = javax.imageio.ImageIO.read(java.io.File(filename))
@@ -401,7 +411,7 @@ class Molecule(object):
                     frame.setSize(300,300)
                     frame.setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE)
                     frame.show()
-                    
+
                 elif sys.platform[:3] == "cli":
                     if filedes:
                         errormessage = ("It is only possible to show the molecule if you "
@@ -420,14 +430,14 @@ class Molecule(object):
                     form.Show()
                     Application.Run(form)
 
-                else:                    
+                else:
                     if not PILtk:
                         errormessage = ("Tkinter or Python Imaging "
                                         "Library not found, but is required for image "
                                         "display. See installation instructions for "
                                         "more information.")
                         raise ImportError, errormessage
-                    
+
                     root = tk.Tk()
                     root.title((hasattr(self, "title") and self.title)
                                or self.__str__().rstrip())
@@ -441,21 +451,21 @@ class Molecule(object):
 
             if filedes:
                 os.close(filedes)
-                os.remove(filename)                
+                os.remove(filename)
 
 class Atom(object):
     """Represent an Indigo Atom.
 
     Required parameters:
        Atom -- an Indigo Atom
-     
+
     Attributes:
         atomicnum, coords, formalcharge
-    
+
     The original Indigo Atom can be accessed using the attribute:
        Atom
     """
-    
+
     def __init__(self, Atom):
         self.Atom = Atom
     @property
@@ -478,14 +488,14 @@ class Smarts(object):
 
     Required parameters:
        smartspattern
-    
+
     Methods:
        findall(molecule)
-    
+
     Example:
     >>> mol = readstring("smi","CCN(CC)CC") # triethylamine
     >>> smarts = Smarts("[#6][#6]") # Matches an ethyl group
-    >>> print smarts.findall(mol) 
+    >>> print smarts.findall(mol)
     [(0, 1), (3, 4), (5, 6)]
 
     The numbers returned are the indices (starting from 0) of the atoms
@@ -501,7 +511,7 @@ class Smarts(object):
 
     def findall(self,molecule):
         """Find all matches of the SMARTS pattern to a particular molecule.
-        
+
         Required parameters:
            molecule
         """
@@ -517,9 +527,9 @@ class Smarts(object):
 
 class MoleculeData(object):
     """Store molecule data in a dictionary-type object
-    
+
     Required parameters:
-      Mol -- an Indigo Mol 
+      Mol -- an Indigo Mol
 
     Methods and accessor methods are like those of a dictionary except
     that the data is retrieved on-the-fly from the underlying Mol.
@@ -583,7 +593,7 @@ class MoleculeData(object):
 
 class Fingerprint(object):
     """A Molecular Fingerprint.
-    
+
     Required parameters:
        fingerprint -- a vector calculated by one of the fingerprint methods
 
@@ -609,6 +619,16 @@ class Fingerprint(object):
     def __str__(self):
         return str(self._buffer_to_int())
 
+def _toint(string):
+    """
+    Some bits sometimes are a character. I haven't found what do they mean,
+    but they break cinfony fingerprints unless taken care of. This functions is just for that.
+    """
+    if string.isdigit():
+        return int(string)
+    else:
+        return 0
+
 def _findbits(fp, bitsperint):
     """Find which bits are set in a list/vector.
 
@@ -651,4 +671,4 @@ def _compressbits(bitvector, wordsize=32):
 if __name__=="__main__": #pragma: no cover
     import doctest
     doctest.testmod()
-    
+
