@@ -3,7 +3,7 @@ import os
 import sys
 import unittest
 
-pybel = indy = ironable = rdk = cdk = obabel = webel = opsin = None
+pybel = indy = ironable = rdk = cdk = webel = opsin = jchem = None
 try:
     import pybel # From Open Babel
 except ImportError:
@@ -13,7 +13,7 @@ try:
 except (RuntimeError, ImportError, KeyError):
     pass
 try:
-    from cinfony import obabel
+    from cinfony import pybel
 except (ImportError, AttributeError, KeyError):
     pass
 try:
@@ -32,6 +32,10 @@ try:
     from cinfony import webel
 except ImportError:
     pass
+try:
+    from cinfony import jchem
+except (NameError, RuntimeError, ImportError, KeyError):
+    pass
 
 try: # Define next() for Jython 2.5
     next
@@ -102,6 +106,15 @@ class TestToolkit(myTestCase):
         self.assertNotEqual(len(self.toolkit.forcefields), 0)
         self.assertNotEqual(len(self.toolkit.fps), 0)
 
+    def testInChI(self):
+        """Test InChI generation"""
+        inchi = self.mols[0].write("inchi").rstrip()
+        inchikey = self.mols[0].write("inchikey").rstrip()
+        self.assertEqual(inchi, "InChI=1S/C4H10/c1-3-4-2/h3-4H2,1-2H3")
+        self.assertEqual(inchikey, "IJDNQMDRQITEOD-UHFFFAOYSA-N")
+        mol = self.toolkit.readstring("inchi", inchi)
+        self.assertEqual("CCCC", mol.write("smi").rstrip())
+
     def FPaccesstest(self):
         # Should raise AttributeError
         return self.mols[0].calcfp().nosuchname
@@ -153,7 +166,7 @@ class TestToolkit(myTestCase):
         newcoords = self.head[0].atoms[0].coords
         self.assertNotEqual(oldcoords, newcoords)
 
-    def notestMake3D(self):
+    def testMake3D(self):
         """Test that 3D coordinate generation does something"""
         mol = self.mols[0]
         mol.make3D()
@@ -354,12 +367,13 @@ M  END
         self.assertEqual(len(self.mols[0].atoms),4)
         
 class TestOBabel(TestToolkit):
-    toolkit = obabel
+    toolkit = pybel
     tanimotoresult = 1/3.
     Natoms = 15
     tpsaname = "TPSA"
     Nfpbits = 32
-    datakeys = ['NSC', 'Comment', 'OpenBabel Symmetry Classes']
+    datakeys = ['NSC', 'Comment', 'OpenBabel Symmetry Classes',
+		'MOL Chiral Flag']
 
     def testFP_FP3(self):
         "Checking the results from FP3"
@@ -370,7 +384,7 @@ class TestOBabel(TestToolkit):
         """Testing unit cell access"""
         mol = next(self.toolkit.readfile("cif", "hashizume.cif"))
         cell = mol.unitcell
-        self.assertAlmostEqual(cell.GetAlpha(), 92.9, 1)
+        self.assertAlmostEqual(cell.GetAlpha(), 93.0, 1)
 
     def testMDcomment(self):
         """Mess about with the comment field"""
@@ -439,6 +453,9 @@ class TestIndigo(TestToolkit):
     def testLocalOpt(self):
         """No forcefields"""
         pass
+    def testMake3D(self):
+        """No forcefields"""
+        pass
 
 
 class TestWebel(TestToolkit):
@@ -588,6 +605,25 @@ class TestCDK(TestToolkit):
         self.assertEqual(len(self.mols[0].atoms), 4)
         self.assertRaises(AttributeError, self.RSaccesstest)
 
+class TestJchem(TestToolkit):
+    toolkit = jchem
+    tanimotoresult = 0.444
+    Natoms = 15
+    tpsaname = "TPSA"
+    Nfpbits = 5
+    datakeys = ['NSC']
+
+    def testLocalOpt(self):
+        """No local opt testing done"""
+        pass
+    def testRSgetprops(self):
+        """Get the values of the properties."""
+        # self.assertAlmostEqual(self.mols[0].exactmass, 58.078, 3)
+        # Only OpenBabel has a working exactmass
+        self.assertAlmostEqual(self.mols[0].molwt, 58.12, 2)
+        self.assertEqual(len(self.mols[0].atoms), 4)
+        self.assertRaises(AttributeError, self.RSaccesstest)
+
 class TestCDKJPype(TestCDK):
     def testDraw(self):
         """No depiction supported I'm afraid"""
@@ -599,7 +635,7 @@ if __name__=="__main__":
 
     lookup = {'cdk': TestCDK, 'obabel':TestOBabel, 'rdk':TestRDKit,
               'webel': TestWebel, 'opsin': TestOpsin, 'indy': TestIndigo,
-              'pybel':TestPybel}
+              'pybel':TestPybel, 'jchem':TestJchem}
     if sys.platform[:4] == "java":
         lookup['obabel'] = TestJybel
         del lookup['rdk']
@@ -607,6 +643,7 @@ if __name__=="__main__":
         lookup['obabel'] = TestIronable
         del lookup['rdk']
         del lookup['cdk']
+        del lookup['jchem']
         del lookup['opsin']
     else:
         lookup['cdk'] = TestCDKJPype
